@@ -138,9 +138,19 @@ window_fill(int window_identifier,
 					 * searching.
 					 */
 					if (opts->ignore_case || (opts->ignore_lower_case && !scrdesc->has_upperchr))
-						str = utf8_nstrstr(str, scrdesc->searchterm);
+					{
+						if (opts->force8bit)
+							str = nstrstr(str, scrdesc->searchterm);
+						else
+							str = utf8_nstrstr(str, scrdesc->searchterm);
+					}
 					else if (opts->ignore_lower_case && scrdesc->has_upperchr)
-						str = utf8_nstrstr_ignore_lower_case(str, scrdesc->searchterm);
+					{
+						if (opts->force8bit)
+							str = nstrstr_ignore_lower_case(str, scrdesc->searchterm);
+						else
+							str = utf8_nstrstr_ignore_lower_case(str, scrdesc->searchterm);
+					}
 					else
 						/* we can use case sensitive searching (binary comparation) */
 						str = strstr(str, scrdesc->searchterm);
@@ -156,7 +166,10 @@ window_fill(int window_identifier,
 						else
 						{
 							lineinfo->mask |= LINEINFO_FOUNDSTR;
-							lineinfo->start_char = utf8len_start_stop(rowstr, str);
+							if (opts->force8bit)
+								lineinfo->start_char = str - rowstr;
+							else
+								lineinfo->start_char = utf8len_start_stop(rowstr, str);
 						}
 
 						str += scrdesc->searchterm_size;
@@ -178,17 +191,11 @@ window_fill(int window_identifier,
 
 			while (str != NULL && npositions < 100)
 			{
-				if (opts->ignore_case || (opts->ignore_lower_case && !scrdesc->has_upperchr))
-					str = utf8_nstrstr(str, scrdesc->searchterm);
-				else if (opts->ignore_lower_case && scrdesc->has_upperchr)
-					str = utf8_nstrstr_ignore_lower_case(str, scrdesc->searchterm);
-				else
-					/* we can use case sensitive searching (binary comparation) */
-					str = strstr(str, scrdesc->searchterm);
+				str = pspg_search(opts, scrdesc, str);
 
 				if (str != NULL)
 				{
-					positions[npositions][0] = utf8len_start_stop(rowstr, str);
+					positions[npositions][0] = opts->force8bit ? str - rowstr : utf8len_start_stop(rowstr, str);
 					positions[npositions][1] = positions[npositions][0] + scrdesc->searchterm_char_size;
 
 					/* don't search more if we are over visible part */
@@ -259,7 +266,7 @@ window_fill(int window_identifier,
 			if (desc->is_expanded_mode)
 			{
 				fix_line_attr_style = effective_row >= desc->border_bottom_row;
-				is_expand_head = is_expanded_header(rowstr, &ei_min, &ei_max);
+				is_expand_head = is_expanded_header(opts, rowstr, &ei_min, &ei_max);
 				if (is_expand_head)
 				{
 					if (scrdesc->first_rec_title_y == -1)
@@ -285,17 +292,35 @@ window_fill(int window_identifier,
 			/* skip first srcx chars */
 			i = srcx;
 			left_spaces = 0;
-			while(i > 0)
+			if (opts->force8bit)
 			{
-				if (*rowstr != '\0' && *rowstr != '\n')
+				while(i > 0)
 				{
-					i -= utf_dsplen(rowstr);
-					rowstr += utf8charlen(*rowstr);
-					if (i < 0)
-						left_spaces = -i;
+					if (*rowstr != '\0' && *rowstr != '\n')
+					{
+						i -= 1;
+						rowstr += 1;
+						if (i < 0)
+							left_spaces = -i;
+					}
+					else
+						break;
+				}
+			}
+			else
+			{
+				while(i > 0)
+				{
+					if (*rowstr != '\0' && *rowstr != '\n')
+					{
+						i -= utf_dsplen(rowstr);
+						rowstr += utf8charlen(*rowstr);
+						if (i < 0)
+							left_spaces = -i;
+					}
+					else
+						break;
 				}
-				else
-					break;
 			}
 
 			/* Fix too hungry cutting when some multichar char is removed */
@@ -356,8 +381,7 @@ window_fill(int window_identifier,
 					{
 						int		htrpos = srcx + i;
 						int		new_attr = active_attr;
-						char	unicode_border[10];
-						int		unicode_border_bytes = 0;
+						bool	print_acs_vline = false;
 						char	column_format;
 
 						column_format = desc->headline_transl != NULL ? desc->headline_transl[htrpos] : ' ';
@@ -367,11 +391,8 @@ window_fill(int window_identifier,
 							if (*(rowstr + left_spaces + bytes) == '|' &&
 									(column_format == 'L' || column_format == 'R' || column_format == 'I'))
 							{
-								strncpy(unicode_border, "\342\224\202", 3);  /* │ */
-								unicode_border_bytes = 3;
+								print_acs_vline = true;
 							}
-							else
-								unicode_border_bytes = 0;
 						}
 
 						if (is_bookmark_row)
@@ -448,7 +469,7 @@ window_fill(int window_identifier,
 							}
 						}
 
-						if (unicode_border_bytes > 0 && bytes > 0)
+						if (print_acs_vline && bytes > 0)
 						{
 							waddnstr(win, rowstr, bytes);
 							rowstr += bytes;
@@ -472,9 +493,9 @@ window_fill(int window_identifier,
 							wattron(win, active_attr);
 						}
 
-						if (unicode_border_bytes > 0)
+						if (print_acs_vline)
 						{
-							waddnstr(win, unicode_border, unicode_border_bytes);
+							waddch(win, ACS_VLINE);
 							bytes = 0;
 							rowstr += 1;
 						}
@@ -511,12 +532,23 @@ window_fill(int window_identifier,
 
 					if (*ptr != '\0')
 					{
-						int len  = utf8charlen(*ptr);
-						i += utf_dsplen(ptr);
-						ptr += len;
-						/* suboptimal logic */
-						if (i <= maxx && rowstr < ptr)
-							bytes += len;
+						if (opts->force8bit)
+						{
+							i += 1;
+							ptr += 1;
+							/* suboptimal logic */
+							if (i <= maxx && rowstr < ptr)
+								bytes += 1;
+						}
+						else
+						{
+							int len  = utf8charlen(*ptr);
+							i += utf_dsplen(ptr);
+							ptr += len;
+							/* suboptimal logic */
+							if (i <= maxx && rowstr < ptr)
+								bytes += len;
+						}
 					}
 					else
 					{
@@ -542,35 +574,33 @@ window_fill(int window_identifier,
 						int		column_format = desc->headline_transl[htrpos];
 						bool	is_top_row = effective_row == desc->border_top_row;
 						bool	is_head_row = effective_row == desc->border_head_row;
-						bool	is_bottom_row = effective_row == desc->border_bottom_row;
 
 						if (column_format == 'd' && *rowstr == '-')
 						{
-							waddnstr(win, "\342\224\200", 3);
+							waddch(win, ACS_HLINE);
 							rowstr += 1;
 							bytes -= 1;
 						}
 						else if (column_format == 'L' && (*rowstr == '+' || *rowstr == '|'))
 						{
 							if (is_head_row)
-								waddnstr(win, "\342\224\234", 3);
+								waddch(win, ACS_LTEE);
 							else if (is_top_row)
-								waddnstr(win, "\342\224\214", 3);
+								waddch(win, ACS_ULCORNER);
 							else /* bottom row */
-								waddnstr(win, "\342\224\224", 3);
+								waddch(win, ACS_LLCORNER);
 
 							rowstr += 1;
 							bytes -= 1;
 						}
 						else if (column_format == 'I' && *rowstr == '+')
 						{
-
 							if (is_head_row)
-								waddnstr(win, "\342\224\274", 3);
+								waddch(win, ACS_PLUS);
 							else if (is_top_row)
-								waddnstr(win, "\342\224\254", 3);
+								waddch(win, ACS_TTEE);
 							else /* bottom row */
-								waddnstr(win, "\342\224\264", 3);
+								waddch(win, ACS_BTEE);
 
 							rowstr += 1;
 							bytes -= 1;
@@ -578,11 +608,11 @@ window_fill(int window_identifier,
 						else if (column_format == 'R' && (*rowstr == '+' || *rowstr == '|'))
 						{
 							if (is_head_row)
-								waddnstr(win, "\342\224\244", 3);
+								waddch(win, ACS_RTEE);
 							else if (is_top_row)
-								waddnstr(win, "\342\224\220", 3);
+								waddch(win, ACS_URCORNER);
 							else /* bottom row */
-								waddnstr(win, "\342\224\230", 3);
+								waddch(win, ACS_LRCORNER);
 
 							rowstr += 1;
 							bytes -= 1;
@@ -590,7 +620,7 @@ window_fill(int window_identifier,
 						}
 						else
 						{
-							int len  = utf8charlen(*rowstr);
+							int len = opts->force8bit ? 1 : utf8charlen(*rowstr);
 
 							waddnstr(win, rowstr, len);
 							rowstr +=len;
@@ -684,6 +714,7 @@ static void
 draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			int maxy, int maxx,				/* size of visible rectangle */
 			int srcy, int srcx,				/* offset to displayed data */
+			Options *opts,
 			DataDesc *desc,
 			attr_t data_attr,				/* colors for data (alphanums) */
 			attr_t line_attr,				/* colors for borders */
@@ -744,7 +775,7 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			if (desc->is_expanded_mode)
 			{
 				fix_line_attr_style = effective_row >= desc->border_bottom_row;
-				is_expand_head = is_expanded_header(rowstr, &ei_min, &ei_max);
+				is_expand_head = is_expanded_header(opts, rowstr, &ei_min, &ei_max);
 			}
 			else
 			{
@@ -760,17 +791,35 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			/* skip first srcx chars */
 			i = srcx;
 			left_spaces = 0;
-			while(i > 0)
+			if (opts->force8bit)
 			{
-				if (*rowstr != '\0' && *rowstr != '\n')
+				while(i > 0)
 				{
-					i -= utf_dsplen(rowstr);
-					rowstr += utf8charlen(*rowstr);
-					if (i < 0)
-						left_spaces = -i;
+					if (*rowstr != '\0' && *rowstr != '\n')
+					{
+						i -= 1;
+						rowstr += 1;
+						if (i < 0)
+							left_spaces = -i;
+					}
+					else
+						break;
+				}
+			}
+			else
+			{
+				while(i > 0)
+				{
+					if (*rowstr != '\0' && *rowstr != '\n')
+					{
+						i -= utf_dsplen(rowstr);
+						rowstr += utf8charlen(*rowstr);
+						if (i < 0)
+							left_spaces = -i;
+					}
+					else
+						break;
 				}
-				else
-					break;
 			}
 
 			/* Fix too hungry cutting when some multichar char is removed */
@@ -849,8 +898,8 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 
 					if (*ptr != '\0' && *ptr != '\n')
 					{
-						int len  = utf8charlen(*ptr);
-						i += utf_dsplen(ptr);
+						int len  = opts->force8bit ? 1 : utf8charlen(*ptr);
+						i += opts->force8bit ? 1 : utf_dsplen(ptr);
 						ptr += len;
 						bytes += len;
 					}
@@ -903,7 +952,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(scrdesc->fix_rows_rows, 0,
 						  scrdesc->rows_rows , scrdesc->fix_cols_cols,
 						  first_data_row + first_row - fix_rows_offset, 0,
-						  desc,
+						  opts, desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  false);
 		}
@@ -915,7 +964,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(0, scrdesc->fix_cols_cols,
 						  scrdesc->fix_rows_rows, size.ws_col - scrdesc->fix_cols_cols,
 						  desc->title_rows + fix_rows_offset, scrdesc->fix_cols_cols + cursor_col,
-						  desc,
+						  opts, desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  true);
 		}
@@ -928,7 +977,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(0, 0,
 						  scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
 						  desc->title_rows + fix_rows_offset, 0,
-						  desc,
+						  opts, desc,
 						  COLOR_PAIR(4) | A_BOLD, 0, COLOR_PAIR(8) | A_BOLD,
 						  false);
 		}
@@ -941,7 +990,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
 						  scrdesc->rows_rows, size.ws_col - scrdesc->fix_cols_cols,
 						  first_data_row + first_row - fix_rows_offset, scrdesc->fix_cols_cols + cursor_col,
-						  desc,
+						  opts, desc,
 						  opts->theme == 2 ? 0 | A_BOLD : 0,
 						  opts->theme == 2 && (desc->headline_transl == NULL) ? A_BOLD : 0,
 						  COLOR_PAIR(8) | A_BOLD,
@@ -956,12 +1005,11 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 			draw_rectange(scrdesc->fix_rows_rows + scrdesc->rows_rows, 0,
 						  scrdesc->footer_rows, scrdesc->maxx,
 						  first_data_row + first_row + scrdesc->rows_rows - fix_rows_offset, footer_cursor_col,
-						  desc,
+						  opts, desc,
 						  COLOR_PAIR(9), 0, 0, true);
 		}
 
 		/* reset */
 		printf("\e[0m\r");
-
 	}
 }
